const _ = require('lodash')
const fs = require('fs')
const path = require('path')
const theo = require('theo')
const gulp = require('gulp')
const serve = require('gulp-serve')
const Color = require('color')
const sass = require('gulp-sass')
const rimraf = require('rimraf')
const runSequence = require('run-sequence')
const autoprefixer = require('gulp-autoprefixer')

const paths = {
  tokens: './tokens',
  generated: './dist',
  sites: './sites'
}

function getPath (p) {
  return path.resolve(__dirname, p)
}

function cleanOutput(output) {
  return output
    .replace(/^    /gm, '')
    .replace(/^\s*\n/gm, '')
    .trim()
}

function clean(p) {
  return function(done) {
    rimraf.sync(getPath(p));
    done();
  }
}

theo.registerFormat('scss', (json, options) => {
  return json.propKeys.map(key => {
    var prop = json.props[key]
    if (prop.type === 'color') {
      prop.value = Color(prop.value).hslString()
    }
    return `$${prop.name}: ${prop.value};`
  }).join('\n')
})

theo.registerFormat('js', (json) => {
  var values = _.map(json.props, prop => {
    var name = prop.name
    var value = prop.value
    if (prop.type === 'color') {
      value = Color(value).hslString()
    }
    switch (typeof value) {
      case 'string':
      default:
        value = `"${value}"`
    }
    return `"${name}": ${value},`
  }).join('\n  ').replace(/,$/, '')
  var output = `
    module.exports = {
      ${values}
    };
  `
  return cleanOutput(output)
})

gulp.task('generate-scss', function () {
  return gulp.src(getPath('./tokens/*.yml'))
    .pipe(theo.plugins.transform('web'))
    .pipe(theo.plugins.format('scss'))
    .pipe(gulp.dest(path.resolve(paths.generated)))
})

gulp.task('generate-css', function () {
  return gulp.src(getPath('./sites/app/css/*.scss'))
    .pipe(sass().on('error', sass.logError))
    .pipe(autoprefixer())
    .pipe(gulp.dest(getPath('./sites/app/css')))
})

gulp.task('generate-js', function () {
  return gulp.src(getPath('./tokens/*.yml'))
    .pipe(theo.plugins.transform('web'))
    .pipe(theo.plugins.format('js'))
    .pipe(gulp.dest(path.resolve(paths.generated)))
})

gulp.task('serve', serve('sites'))

gulp.task('watch', ['serve'], function () {
  gulp.watch(getPath('./tokens/*.yml'), ['generate-js', 'generate-scss'])
  gulp.watch(getPath('./sites/app/css/*.scss'), ['generate-css'])
})

gulp.task('clean', clean(paths.generated))

gulp.task('default', function (done) {
  runSequence('clean', ['generate-js', 'generate-scss'], ['generate-css'], done)
})
